#ifndef _RECEIVER_H_
#define _RECEIVER_H_

/*
 * Copyright (C) 2006 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * \file receiver.h
 * \brief Audio data receiver.
 *
 * \verbatim
 *
 * $Log$
 *
 * \endverbatim
 */

#include <sys/socket.h>
#include "config.h"
#include "station.h"

#ifndef MP3_BUFSIZ
#if defined (AT91SAM9260_EK)
#define MP3_BUFSIZ          1048576
#else
#define MP3_BUFSIZ          (3 * MAINBUF_SIZE)
#endif
#endif

#ifndef MAX_WAITSTREAM
#define MAX_WAITSTREAM      20
#endif

#define RSTAT_IDLE      0
#define RSTAT_START     1
#define RSTAT_RUNNING   2
#define RSTAT_STOP      3

#define RECEIVER_TYPE_SHOUTCAST 1

typedef struct _RECEIVERINFO {
    u_int ri_status;       /*!< Current status of the stream. SSTAT_. */
    HANDLE ri_tid;
    HANDLE ri_cmdevt;
    HANDLE ri_stsevt;
    u_char *ri_buff;
    u_int ri_wpos;
    u_int ri_rpos;
    u_int ri_avail;
    HANDLE ri_wrbque;
    HANDLE ri_rdbque;
    STATIONINFO *ri_sip;
    void *ri_bcast;
} RECEIVERINFO;

__BEGIN_DECLS
/* Prototypes */

extern RECEIVERINFO * ReceiverCreate(u_int type);
extern u_int ReceiverThreadStatus(RECEIVERINFO *rip);
extern int ReceiverStart(RECEIVERINFO *rip, STATIONINFO *sip);
extern int ReceiverStop(RECEIVERINFO *rip);

__END_DECLS
/* End of prototypes */
#endif
