/*!
 * Copyright (C) 2006 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log$
 */

#include <cfg/os.h>
#include <cfg/clock.h>
#include <dev/board.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>

#include <sys/version.h>
#include <sys/confnet.h>
#include <sys/atom.h>
#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <sys/socket.h>

#include <hxmp3/mp3dec.h>

#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <pro/dhcp.h>

#include "at73dac.h"
#include "config.h"
#include "player.h"
#include "shoutcast.h"

/*
 * Maximum segment size, choose 536 up to 1460. Note, that segment 
 * sizes above 536 may result in fragmented packets. Remember, that 
 * Ethernut doesn't support TCP fragmentation.
 */
#if defined(AT91SAM9260_EK)
static u_short mss = 1460;
#else
static u_short mss = 536;
#endif

/*
 * Socket receive timeout.
 */
static u_long rx_to = 5000;

/*
 * TCP buffer size.
 */
#if defined(AT91SAM9260_EK)
static u_short tcpbufsiz = 11680;
#else
static u_short tcpbufsiz = 3216;
#endif

/*
 * Get line from naked TCP stream.
 */
static int GetLine(TCPSOCKET * sock, char * line, u_short size)
{
    int rc = 0;
    u_char to_cnt = 0;
    int got;
    char *cp = line;

    if (size > 0) {
        for (;;) {
            if ((got = NutTcpReceive(sock, cp, 1)) <= 0) {
                if (got == 0 && to_cnt++ < 10)
                    continue;
                rc = got;
                printf("[GL-TO]");
                break;
            }
            if (*cp == '\n') {
                *cp = 0;
                break;
            }
            if (*cp != '\r' && rc < (int) size) {
                rc++;
                cp++;
            }
        }
    }
    return rc;
}

static int PutString(TCPSOCKET * sock, char * str)
{
    u_short len = strlen(str);
    u_short n;
    int c;

    for (n = 0; n < len; n += c)
        if ((c = NutTcpSend(sock, str + n, len - n)) < 0)
            return -1;
    return len;
}


/*!
 * \brief Create station information from the header of a connected radio station.
 */
static STATIONINFO * StationCreate(TCPSOCKET * sock, STATIONCONF * radio)
{
    STATIONINFO *sip;
    char *line;
    char *cp;
    int err = -1;

    if ((line = malloc(256)) == NULL) {
        return NULL;
    }

    /*
     * Send the HTTP request.
     */
    strcpy(line, "GET /");
    if (proxy.proxy_ip) {
        sprintf(line, "GET http://%s:%u/", inet_ntoa(radio->rs_ip), radio->rs_port);
    }
    else {
        strcpy(line, "GET /");
    }
    if (radio->rs_url) {
        strcat(line, radio->rs_url);
    }
    strcat(line, " HTTP/1.0\r\n");
    PutString(sock, line);

    sprintf(line, "Host: %s\r\n", inet_ntoa(radio->rs_ip));
    PutString(sock, line);

    PutString(sock, "User-Agent: WinampMPEG/2.7\r\n" /* */
        "Accept: */*\r\n"  /* */
        "Icy-MetaData: 1\r\n"  /* */
        "Connection: close\r\n\r\n");

    /*
     * Create stream info structure.
     */
    if ((sip = malloc(sizeof(STATIONINFO))) == NULL) {
        free(line);
        return NULL;
    }
    memset(sip, 0, sizeof(STATIONINFO));
    sip->si_sock = sock;

    /*
     * Receive the HTTP header.
     */
    if (GetLine(sock, line, 256) > 5) {
        printf("%s\n", line);
        if (strncmp(line, "ICY", 3) == 0) {
            if (atoi(line + 4) == 200) {
                /*
                 * Shoutcast MP3 detected. This check is probably not sufficient.
                 */
                sip->si_stype = STATION_TYPE_SHOUTCAST;
                sip->si_ptype = PLAYER_TYPE_MP3;

                for (;;) {
                    if ((err = GetLine(sock, line, 256)) <= 0) {
                        break;
                    }
                    err = 0;
                    printf("%s\n", line);
                    if (strncmp(line, "icy-name:", 9) == 0) {
                        cp = line + 9;
                        while (*cp && *cp == ' ') {
                            cp++;
                        }
                        if (*cp && sip->si_name == 0) {
                            sip->si_name = malloc(strlen(cp) + 1);
                            strcpy(sip->si_name, cp);
                            printf("%s\n", cp);
                        }
                    } else if (strncmp(line, "icy-genre:", 10) == 0) {
                        cp = line + 10;
                        while (*cp && *cp == ' ') {
                            cp++;
                        }
                        if (*cp && sip->si_genre == 0) {
                            sip->si_genre = malloc(strlen(cp) + 1);
                            strcpy(sip->si_genre, cp);
                        }
                    } else if (strncmp(line, "icy-metaint:", 12) == 0) {
                        sip->si_metaint = atol(line + 12);
                    }
                    else if (strncmp(line, "icy-br:", 7) == 0) {
                        sip->si_bitrate = atol(line + 7);
                    }
                }
            } else {
                puts(line);
            }
        }
    }

    free(line);

    if (err) {
        printf("\nScan failed.\n");
        free(sip);
        sip = NULL;
    }
    else {
        printf("\n%s %ukbps %s ", inet_ntoa(radio->rs_ip), sip->si_bitrate, sip->si_name);
    }

    return sip;
}

/*
 * Disconnect from a radio station.
 */
void StationDisconnect(STATIONINFO *sip)
{
    printf("Disconnecting\n");
    if (sip) {
        if (sip->si_sock) {
            NutTcpCloseSocket(sip->si_sock);
        }
        free(sip);
    }
}

/*!
 * \brief Connect to a radio station.
 */
STATIONINFO *StationConnect(STATIONCONF * radio)
{
    STATIONINFO *sip = NULL;
    TCPSOCKET *sock = NULL;
    int err = -1;
    int cr;


    /* Create a TCP socket. */
    if ((sock = NutTcpCreateSocket()) != NULL) {
        /* Set socket options. Failures are ignored. */
        if (NutTcpSetSockOpt(sock, TCP_MAXSEG, &mss, sizeof(mss)))
            printf("Sockopt MSS failed\n");
        if (NutTcpSetSockOpt(sock, SO_RCVTIMEO, &rx_to, sizeof(rx_to)))
            printf("Sockopt TO failed\n");
        if (NutTcpSetSockOpt(sock, SO_RCVBUF, &tcpbufsiz, sizeof(tcpbufsiz)))
            printf("Sockopt rxbuf failed\n");

        /* Connect the stream server. */
        printf("[CNCT %s:%u", inet_ntoa(radio->rs_ip), radio->rs_port);
        if (proxy.proxy_ip) {
            printf(" via %s:%u]", inet_ntoa(proxy.proxy_ip), proxy.proxy_port);
            cr = NutTcpConnect(sock, proxy.proxy_ip, proxy.proxy_port);
        }
        else {
            putchar(']');
            cr = NutTcpConnect(sock, radio->rs_ip, radio->rs_port);
        }
        if (cr) {
            printf("[CERR=%d]\n", NutTcpError(sock));
        }
        else {
            printf("[CNCTD]");
            /* Process header from server. */
            if ((sip = StationCreate(sock, radio)) != NULL) {
                err = 0;    /* Success. */
            }
        }
    }

    /* Release resources in case of any error. */
    if (err) {
        if (sip) {
            StationDisconnect(sip);
            sip = NULL;
        }
        else if (sock) {
            NutTcpCloseSocket(sock);
        }
    }
    return sip;
}
